@@ -36,17 +36,17 @@ if [ $? -eq 0 ]; then
             # We are going to build a universal (ppc32, ppc64, x86_32, x86_64)
             #    combined binary, so we only need 1 host type
             os_release=`uname -r | $AWK -F . '{ printf "%s", $1 }'`
-	    if [ $os_release -ge 11 ]; then
-		# GCC deprecated, so GCC thinks this is 10.7...
-		gcc_os_name=`echo $gcc_os_name|sed "s/11/$os_release/"`
-	    fi
+	    	if [ $os_release -ge 11 ]; then
+				# GCC deprecated, will always be 4.2.1 going forward. Strip extra versioning off
+				gcc_os_name=`echo $gcc_os_name|awk -F. '{ print $1 }'`
+	    	fi
             if [ $os_release -ge 8 ]; then
                 # This is 10.4 so Universal builds possible
                 gcc_host="${gcc_os_vendor}-${gcc_os_name}"
                 # Note, for BSDish platforms, gcc_os_name includes a version
             else
-		# Prior to Tiger and gcc4, things were a mess...
-		gcc_host="${gcc_cpu_type}-${gcc_os_vendor}"
+				# Prior to Tiger and gcc4, things were a mess...
+				gcc_host="${gcc_cpu_type}-${gcc_os_vendor}"
                 # Note, gcc_os_vendor in this case is actually gcc_os_name above
 	    fi
 	    ;;
