@@ -16,8 +16,8 @@ function [img,dims,scales,bpp,endian] = read_avw_complex(fname)
 %            SAVE_AVW_IMG, SAVE_AVW_COMPLEX
 %   
 
-command=sprintf('sh -c ". ${FSLDIR}/etc/fslconf/fsl.sh; ${FSLDIR}/bin/fslcomplex -realcartesian %s %s %s "\n',fname,[fname,'R'],[fname,'I']);
-system(command);
+command=sprintf('${FSLDIR}/bin/fslcomplex -realcartesian %s %s %s',fname,[fname,'R'],[fname,'I']);
+call_fsl(command);
 
 [imgr,dims,scales,bpp,endian]=read_avw([fname,'R']);
 [imgi,dims,scales,bpp,endian]=read_avw([fname,'I']);
